/***************************************************************************
 *   Copyright (C) 2004 by Andr Stein                                     *
 *   stone@steinsoft.net                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef SGLSLPROGRAM_H
#define SGLSLPROGRAM_H

#include "sglslextension.h"


/**
represents a GLslang program in opengl that uses vertex/fragment shaders
represented by glslShader

@author Andr Stein
*/
class sglslProgram
{
public:
	sglslProgram();
	sglslProgram(const char* vsFilename, const char* fsFilename = 0);
	~sglslProgram();
    
	void createFromFile(const char* vsFilename, const char* fsFilename = 0);
	void create(const char* vsSource, const char* fsSource = 0);
	void destroy();
	void bind();
	
	/** inline functions to set uniforms **/
	void setUniform(const char* name, int x)
	 { pglUniform1iARB(pglGetUniformLocationARB(handle, name), x); }
	void setUniform(const char* name, int x, int y) 
	{ pglUniform2iARB(pglGetUniformLocationARB(handle, name), x, y); }
	void setUniform(const char* name, int x, int y, int z)
	 { pglUniform3iARB(pglGetUniformLocationARB(handle, name), x, y, z); }
	void setUniform(const char* name, int x, int y, int z, int q)
	 { pglUniform4iARB(pglGetUniformLocationARB(handle, name), x, y, z, q); }
	void setUniform(const char* name, float x, float y, float z, float q)
	 { pglUniform4fARB(pglGetUniformLocationARB(handle, name), x, y, z, q); }
	void setUniform(const char* name, float x, float y, float z)
	 { pglUniform3fARB(pglGetUniformLocationARB(handle, name), x, y, z); }
	void setUniform(const char* name, float x, float y)
	 { pglUniform2fARB(pglGetUniformLocationARB(handle, name), x, y); }
	void setUniform(const char* name, float x)
	 { pglUniform1fARB(pglGetUniformLocationARB(handle, name), x); }
	void setUniform(const char* name, int size, float* matrix, bool transpose = false);
	
	
	static void unbind() { pglUseProgramObjectARB(0); }
protected:
	GLenum handle;
	
	const string loadShader(const char* filename);

};

#endif
